Writer

Struct Writer 

pub struct Writer { /* private fields */ }
An incremental writer to a Vec<u8>, with support for delayed labels.

One of the main points of interest is u32_le and the like, which read the relevant primitives from the slice. In these docs, they are abbreviated to a single function for simplicity.

The other point of interest is the place and label functions. These allow writing delayed data that cannot be known before other data has been written; in particular sizes and offsets.

Supported primitives are u8..=u128, i8..=i128, f32, f64.

The functions are suffixed with either _le or _be, for endianness. To use unsuffixed versions, import either the Le or Be trait.

Implementations

Source

impl Writer

pub fn new() -> Self

Constructs a new Writer.

pub fn finish(self) -> Result<Vec<u8>>

Finalizes all delayed labels and returns the resulting Vec<u8>.

Returns any error returned by the delays, which is usually if a label is not defined or is too large to fit in its slot.

pub fn len(&self) -> usize

Returns the number of bytes written so far, including delayed ones.

pub fn is_empty(&self) -> bool

Returns whether any bytes have been written so far.

pub fn reserve(&mut self, size: usize)

Calls Vec::reserve on the underlying Vec.

pub fn capacity(&self) -> usize

Returns the capacity of the underlying Vec.

pub fn T(&mut self, val: T)

Write a primitive from the input.

pub fn slice(&mut self, data: &[u8])

Writes some data.

pub fn array<const N: usize>(&mut self, data: [u8; N])

Writes some data.

This function is redundant, and exists only for symmetry.

pub fn place(&mut self, label: Label)

Places a label at the current position, so it can be referenced with labelN.

Placing the same label twice results in a panic.

pub fn here(&mut self) -> Label

Creates and places a label at the current position.

pub fn labelN(&mut self, l: Label)

Write the address of a label.

finish will throw an error if the resulting address does not fit in the type.

pub fn diffN(&mut self, start: Label, end: Label)

Write the difference between two labels.

finish will throw an error if the resulting value does not fit in the type.

pub fn delay<const N: usize, F>(&mut self, cb: F)
where F: FnOnce(&DelayContext<'_>) -> Result<[u8; N], Box<dyn Error + Send + Sync>> + 'static,

Writes some bytes to be filled in later.

The given closure is called with a function that allows looking up labels. Other kinds of state are not currently officially allowed.

pub fn ptrN(&mut self) -> Writer

Creates a new Writer and delays a pointer to it.

This is a shorthand for the common pattern of { let mut g = Writer::new(); f.delayN(g.here()); g }.

pub fn align(&mut self, size: usize)

Writes null bytes until the length is a multiple of size.

pub fn append(&mut self, other: Writer)

Concatenates two Writers, including labels.